"""
A configuration manager using wxPython and ConfigParser.

Written by Chris Norman.
"""

import wx, confmanager.confframe
from wx.lib import intctrl
from wx.lib.agw import floatspin
from collections import OrderedDict
from sys import version_info
from confmanager.errors import *

if version_info.major == 2:
 BASESTRING = basestring
else:
 BASESTRING = str

class ConfManager(object):
 """Use this class to make your configuration objects."""
 
 def __init__(self, title):
  """
  __init__(self, title)
  title will become the title of any GUI created using self.get_gui()
  """
  self.title = title # Store it away for future use
  self.helpFunc = lambda event: wx.MessageBox('There is no help for this configuration screen', 'No help available') # The help function to be bound to the help button on the settings pages.
  self.updateFunc = lambda self, section, option, value: None # The value to be called when options change.
  # Set up the controls which will be used for each type of value:
  self.control_types = OrderedDict()
  self.control_types[bool] = wx.CheckBox
  self.control_types[float] = floatspin.FloatSpin
  self.control_types[int] = intctrl.IntCtrl
  self.control_types[BASESTRING] = wx.TextCtrl
  self.config = OrderedDict() # The saved options.
  self.section_names = {} # The friendly names for all sections.
 
 def add_section(self, section, friendly_name = None):
  """Adds a section and optionally gives it a friendly name.."""
  if not isinstance(section, BASESTRING): # Make sure the user isn't expecting to use something stupid as a key.
   raise ValueError(section)
  # See if we've got this section already:
  if section in self.config:
   raise DuplicateSectionError(section) # Yep... Kick off.
  else:
   self.config[section] = OrderedDict() # Nope... Ad it
   if friendly_name == None:
    friendly_name = section.title()
   if '&' not in friendly_name:
    friendly_name = '&' + friendly_name
   self.section_names[section] = friendly_name
 
 def set(self, section, option, value = None, title = None, validate = None, help = None, control = None, args = None, kwargs = None, include = None):
  """
  set(self, section, option, value = None, title = None, validate = lambda value: None, help = '', control = None, args = [], kwargs = {}, include = True)
  Store the value to the configuration with all it's associated data.
  
  * title is the friendly name for the value (will be used for textual labels etc).
  * validate is a lambda which will be called by ConfigFrame.onOk when checking all the values. Should return None if the value is ok, or an error message as a string.
  * help is the help message which will be tacked onto the control in the GUI.
  * control is the control which will be used in the GUI.
  * args is the arguments that will be passed to the control in addition to the ones needed by ConfFrame.
  * kwargs is the keyword arguments which will be passed to the control when it's created by self.get_gui.
  * include tells the GUI whether or not to include this value in the resulting configuration screens.
  
  Also alerts of a change, using self.updateFunc.
  
  """
  if self.has_option(section, option):
   oldValue = self.get(section, option) # Store for comparison.
   v = self.config[section][option] # Don't alter anything but the value.
  else:
   oldValue = value # It's new, don't alert.
   v = {} # The entry for self.config[section][value].
  if title != None:
   v['title'] = title
  else:
   v['title'] = v.get('title', option.title()) # Set default capitalised title
  if help != None:
   v['help'] = help
  else:
   v['help'] = v.get('help', '')
  if validate != None:
   v['validate'] = validate
  else:
   v['validate'] = v.get('validate', lambda value: None)
  if control != None:
   v['control'] = control
  else:
   ctrl = None
   for x, y in self.control_types.items():
    if issubclass(type(value), x):
     ctrl = y
     break
   else:
    raise TypeError('No control provided, and none found for type of value: %s.' % repr(value))
   v['control'] = v.get('control', ctrl)
  if args != None:
   v['args'] = args
  else:
   v['args'] = v.get('args', [])
  if kwargs != None:
   v['kwargs'] = kwargs
  else:
   v['kwargs'] = v.get('kwargs', {})
  if include != None:
   v['include'] = include
  else:
   v['include'] = v.get('include', True)
  v['value'] = value
  # Add everything to self.config:
  self.config[section][option] = v
  if value != oldValue: # Then call self.updateFunc
   self.updateFunc(self, section, option, value)
 
 def remove_section(self, section):
  """Removes sections."""
  del self.config[section]
 
 def remove_option(self, section, option):
  """Removes options."""
  del self.config[section][option]
 
 def toggle(self, section, option):
  """Toggles option in section."""
  self.set(section, option, not self.get(section, option))
 
 def get_spec(self, section, option):
  """Gets the real deal from self.config. Arguments are the same as if you were making a call to self.get."""
  return self.config[section][option] # Yaye!
 
 def get_gui(self):
  """Gets a GUI based on this object."""
  return confframe.ConfFrame(self) # Give up the goods to ConfFrame
 
 def get(self, section, option, default = None):
  """Returns the option's value converted into it's intended type. If default is specified, return that on failure, else raise NoOptionError."""
  if self.has_section(section):
   try:
    return self.config[section][option].get('value', None)
   except KeyError:
    if default == None:
     raise NoOptionError(option)
    else:
     return default
  else:
   raise NoSectionError(section)
 
 def has_option(self, section, option):
  """See if option is in the config."""
  return option in self.config.get(section, {})
 
 def has_section(self, section):
  """Check if we have section in self.config."""
  return section in self.config
 
 def sections(self):
  return self.config.keys()
 
 def options(self, section):
  """Returns the options in section."""
  return self.config[section].keys()
 
 def get_dump(self):
  """Returns options and values."""
  res = {}
  for section in self.sections():
   res[section] = {}
   for option in self.options(section):
    res[section][option] = self.get(section, option)
  return res
