"""
The frame to be used by ConfManager objects.
"""

import wx
import wx.lib.sized_controls as sc

class ConfFrame(wx.Frame):
 """This is the class that ConfManager.get_gui() makes it's frames from."""
 def __init__(self, config):
  """Called by ConfManager.get_gui."""
  self.config = config # The config object which this frame is based on
  self._controls = dict() # The underlying controls, used for self.onOk
  super(ConfFrame, self).__init__(None, title = config.title) # Create the frame wxPython style
  self.pane = wx.Panel(self) # Used as a parent for all the controls which are created
  self.notebook = wx.Notebook(self.pane) # The main notebook
  for section in self.config.config:
   # Run through each section and make controls accordingly:
   page = sc.SizedPanel(self.notebook) # Make a page for the notebook
   page.SetSizerType('form') # 2-column grid
   self._controls[section] = dict() # Prepare for control insertion
   for option, o in self.config.config[section].items():
    # Run through all the options in the current section and do the business:
    if not o['include']:
     continue # This option is hidden, so skip over it.
    l = '%s%s' % ('' if '&' in o['title'] else '&', o['title'])
    wx.StaticText(page, label = l) # Make a label
    control = o['control'](page, *o['args'], **o['kwargs']) # Make the control
    control.SetHelpText(o['help']) # Set the help text for the current control
    if isinstance(control, wx.CheckBox) or isinstance(control, wx.Button):
     control.SetLabel(l) # Set labels for buttons and checkboxes
    self._controls[section][option] = control # Add the control to the database of controls
    control.SetValue(o['value'])
   self.notebook.AddPage(page, self.config.section_names[section]) # Finally add the page we created
  s = wx.BoxSizer(wx.VERTICAL)
  s.Add(self.notebook, 1, wx.GROW)
  self.pane.SetSizerAndFit(s)
  self.notebook.Fit() # Fit the notebook to the frame
  self.Maximize(True) # I hate windows which aren't maximised when I open them!
  self.SetTitle(self.config.title) # Finally use the first property from the ConfManager object
 
 def Show(self, value = True):
  """Before we show the frame, we need to add the finishing touches:"""
  self.Bind(wx.EVT_BUTTON, lambda event: self.Close(True), wx.Button(self.pane, label = '&Cancel')) # Add a cancel button
  ok = wx.Button(self.pane, label = '&OK') # Add the ok button
  ok.SetDefault() # When the user presses enter, this is the button which gets activated
  self.Bind(wx.EVT_BUTTON, self.onOk, ok) # Bind it
  self.Bind(wx.EVT_BUTTON, self.apply, wx.Button(self.pane, label = '&Apply')) # Apply the configuration changes to the ConfigParser object, but don't close the window
  help = wx.Button(self.pane, label = '&Help')
  if not callable(self.config.helpFunc):
   help.Disable()
  help.Bind(wx.EVT_BUTTON, self.config.helpFunc) # Add a help button
  return super(ConfFrame, self).Show(value) # We're done here
 
 def apply(self, event = None):
  """Before self.onOk closes the window, it calls this function to sync the config changes from the GUI back to self.config."""
  for section in self.config.sections():
   # Run through the sections to check all the option values:
   for option, o in self.config.config[section].items():
    # Check the actual values against the validators and complain if necessary:
    if not o['include']:
     continue # This value is hidden, so there's no control for it.
    control = self._controls[section][option] # Get the actual control for GetValue
    try:
     value = type(o['value'])(control.GetValue()) # Try and convert the value
    except ValueError as msg:
     self.displayError(section, option, str(msg)) # Woops, something went wrong
     return False # Tells self.onOk not to close the window
    problem = None # Set up the problem variable.
    try:
     problem = o['validate'](value) # See if it passes the test
    except Exception as e:
     problem = str(e) # The lambda raised an exception.
    if problem:
     self.displayError(section, option, problem) # It didn't
     return False # Tells self.onOk not to close the window
    self.config.set(section, option, value) # All clear
  return True # Let self.onOk know it can close the window
 
 def onOk(self, event):
  """Checks to see if all the entered values match up to expectation."""
  if self.apply():
   self.Close(True) # Only close if apply was successful to avoid losing changes
 
 def displayError(self, section, option, message):
  """
  Complain about something that was entered into a field on the notebook.
  
  Made this into a seporate function, because it was used more than once.
  
  displayError(self, section, option, message)
  Arguments are like those given to ConfManager.get, with the additional message as text.
  
  Shows message as an alert box, and moves focus back to the control, jumping pages in the notebook if necessary.
  """
  self.notebook.SetSelection(self.config.sections().index(section)) # Jump pages, using the position of section to find the relevant page number
  self._controls[section][option].SetFocus() # Set focus on the offending control
  wx.MessageBox(message, 'Error') # Display the error
